@@ -125,9 +125,9 @@ fn generate_address(seed: &Seed, account_index: u32, address_index: u32, interna
     let chain = Chain::from_u32_hardened(vec![44, 4218, account_index, internal as u32, address_index]);
     let public_key = seed
         .derive(Curve::Ed25519, &chain)?
-        .secret_key()?
+        .secret_key()
         .public_key()
-        .to_compressed_bytes();
+        .to_bytes();
     // Hash the public key to get the address
     let result = Blake2b256::digest(&public_key)
         .try_into()
@@ -144,7 +144,7 @@ pub async fn search_address(
     range: Range<usize>,
     address: &Address,
 ) -> Result<(usize, bool)> {
-    let addresses = GetAddressesBuilder::new(&seed)
+    let addresses = GetAddressesBuilder::new(seed)
         .with_bech32_hrp(bech32_hrp.to_owned())
         .with_account_index(account_index)
         .with_range(range.clone())
