@@ -12,6 +12,7 @@ use crate::{
 use bee_common::packable::Packable;
 use bee_message::{
     constants::INPUT_OUTPUT_COUNT_MAX,
+    payload::Payload,
     prelude::{
         Address, Ed25519Address, Message, MessageBuilder, MessageId, Parents, TransactionId, UtxoInput,
         ED25519_ADDRESS_LENGTH,
@@ -22,10 +23,12 @@ use bee_pow::providers::{
     NonceProvider, NonceProviderBuilder,
 };
 use bee_rest_api::types::{
-    dtos::{MessageDto, PeerDto, ReceiptDto},
+    body::SuccessBody,
+    dtos::{LedgerInclusionStateDto, MessageDto, PeerDto, ReceiptDto},
     responses::{
-        BalanceAddressResponse, InfoResponse as NodeInfo, MilestoneResponse as MilestoneResponseDto, OutputResponse,
-        ReceiptsResponse, TipsResponse, TreasuryResponse, UtxoChangesResponse as MilestoneUTXOChanges,
+        BalanceAddressResponse, InfoResponse as NodeInfo, MessageResponse, MilestoneResponse as MilestoneResponseDto,
+        OutputResponse, PeersResponse, ReceiptsResponse, SubmitMessageResponse, TipsResponse, TreasuryResponse,
+        UtxoChangesResponse as MilestoneUTXOChanges,
     },
 };
 use crypto::{
@@ -469,7 +472,7 @@ impl Client {
     }
 
     /// Get a node candidate from the synced node pool.
-    pub(crate) async fn get_node(&self) -> Result<Node> {
+    pub async fn get_node(&self) -> Result<Node> {
         if let Some(primary_node) = &self.node_manager.primary_node {
             return Ok(primary_node.clone());
         }
@@ -685,11 +688,8 @@ impl Client {
 
         let path = "api/v1/info";
         url.set_path(path);
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: NodeInfo,
-        }
-        let resp: ResponseWrapper = crate::node_manager::HttpClient::new()
+
+        let resp: SuccessBody<NodeInfo> = crate::node_manager::HttpClient::new()
             .get(Node { url, jwt }, GET_API_TIMEOUT)
             .await?
             .json()
@@ -713,26 +713,20 @@ impl Client {
     /// GET /api/v1/peers endpoint
     pub async fn get_peers(&self) -> Result<Vec<PeerDto>> {
         let path = "api/v1/peers";
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: Vec<PeerDto>,
-        }
-        let resp: ResponseWrapper = self
+
+        let resp: SuccessBody<PeersResponse> = self
             .node_manager
             .get_request(path, None, self.get_timeout(Api::GetPeers))
             .await?;
 
-        Ok(resp.data)
+        Ok(resp.data.0)
     }
 
     /// GET /api/v1/tips endpoint
     pub async fn get_tips(&self) -> Result<Vec<MessageId>> {
         let path = "api/v1/tips";
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: TipsResponse,
-        }
-        let resp: ResponseWrapper = self
+
+        let resp: SuccessBody<TipsResponse> = self
             .node_manager
             .get_request(path, None, self.get_timeout(Api::GetTips))
             .await?;
@@ -755,25 +749,16 @@ impl Client {
         } else {
             self.get_timeout(Api::PostMessageWithRemotePow)
         };
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: MessageIdWrapper,
-        }
-        #[derive(Debug, Serialize, Deserialize)]
-        struct MessageIdWrapper {
-            #[serde(rename = "messageId")]
-            message_id: String,
-        }
 
         #[cfg(not(feature = "pow-fallback"))]
-        let resp: ResponseWrapper = self
+        let resp: SuccessBody<SubmitMessageResponse> = self
             .node_manager
             .post_request_bytes(path, timeout, &message.pack_new(), local_pow)
             .await?;
 
         #[cfg(feature = "pow-fallback")]
         // fallback to local PoW if remote PoW fails
-        let resp: ResponseWrapper = match self
+        let resp: SuccessBody<SubmitMessageResponse> = match self
             .node_manager
             .post_request_bytes(path, timeout, &message.pack_new(), local_pow)
             .await
@@ -849,18 +834,9 @@ impl Client {
             self.get_timeout(Api::PostMessageWithRemotePow)
         };
         let message_dto = MessageDto::from(message);
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: MessageIdWrapper,
-        }
-        #[derive(Debug, Serialize, Deserialize)]
-        struct MessageIdWrapper {
-            #[serde(rename = "messageId")]
-            message_id: String,
-        }
 
         // fallback to local PoW if remote PoW fails
-        let resp: ResponseWrapper = match self
+        let resp: SuccessBody<SubmitMessageResponse> = match self
             .node_manager
             .post_request_json(path, timeout, serde_json::to_value(message_dto)?, local_pow)
             .await
@@ -942,11 +918,7 @@ impl Client {
             hex::encode(output_id.output_id().index().to_le_bytes())
         );
 
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: OutputResponse,
-        }
-        let resp: ResponseWrapper = self
+        let resp: SuccessBody<OutputResponse> = self
             .node_manager
             .get_request(path, None, self.get_timeout(Api::GetOutput))
             .await?;
@@ -1009,12 +981,8 @@ impl Client {
     /// Get the milestone by the given index.
     pub async fn get_milestone(&self, index: u32) -> Result<MilestoneResponse> {
         let path = &format!("api/v1/milestones/{}", index);
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: MilestoneResponseDto,
-        }
 
-        let resp: ResponseWrapper = self
+        let resp: SuccessBody<MilestoneResponseDto> = self
             .node_manager
             .get_request(path, None, self.get_timeout(Api::GetMilestone))
             .await?;
@@ -1033,11 +1001,8 @@ impl Client {
     /// Get the milestone by the given index.
     pub async fn get_milestone_utxo_changes(&self, index: u32) -> Result<MilestoneUTXOChanges> {
         let path = &format!("api/v1/milestones/{}/utxo-changes", index);
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: MilestoneUTXOChanges,
-        }
-        let resp: ResponseWrapper = self
+
+        let resp: SuccessBody<MilestoneUTXOChanges> = self
             .node_manager
             .get_request(path, None, self.get_timeout(Api::GetMilestone))
             .await?;
@@ -1049,45 +1014,28 @@ impl Client {
     /// Get all receipts.
     pub async fn get_receipts(&self) -> Result<Vec<ReceiptDto>> {
         let path = &"api/v1/receipts";
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: ReceiptsResponseWrapper,
-        }
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ReceiptsResponseWrapper {
-            receipts: ReceiptsResponse,
-        }
-        let resp: ResponseWrapper = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
 
-        Ok(resp.data.receipts.0)
+        let resp: SuccessBody<ReceiptsResponse> = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
+
+        Ok(resp.data.receipts)
     }
 
     /// GET /api/v1/receipts/{migratedAt} endpoint
     /// Get the receipts by the given milestone index.
     pub async fn get_receipts_migrated_at(&self, milestone_index: u32) -> Result<Vec<ReceiptDto>> {
         let path = &format!("api/v1/receipts/{}", milestone_index);
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: ReceiptsResponseWrapper,
-        }
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ReceiptsResponseWrapper {
-            receipts: ReceiptsResponse,
-        }
-        let resp: ResponseWrapper = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
 
-        Ok(resp.data.receipts.0)
+        let resp: SuccessBody<ReceiptsResponse> = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
+
+        Ok(resp.data.receipts)
     }
 
     /// GET /api/v1/treasury endpoint
     /// Get the treasury output.
     pub async fn get_treasury(&self) -> Result<TreasuryResponse> {
         let path = "api/v1/treasury";
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: TreasuryResponse,
-        }
-        let resp: ResponseWrapper = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
+
+        let resp: SuccessBody<TreasuryResponse> = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
 
         Ok(resp.data)
     }
@@ -1096,12 +1044,9 @@ impl Client {
     /// Returns the included message of the transaction.
     pub async fn get_included_message(&self, transaction_id: &TransactionId) -> Result<Message> {
         let path = &format!("api/v1/transactions/{}/included-message", transaction_id);
-        #[derive(Debug, Serialize, Deserialize)]
-        struct ResponseWrapper {
-            data: MessageDto,
-        }
-        let resp: ResponseWrapper = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
-        Ok(Message::try_from(&resp.data)?)
+
+        let resp: SuccessBody<MessageResponse> = self.node_manager.get_request(path, None, GET_API_TIMEOUT).await?;
+        Ok(Message::try_from(&resp.data.0)?)
     }
     /// Reattaches messages for provided message id. Messages can be reattached only if they are valid and haven't been
     /// confirmed for a while.
@@ -1313,7 +1258,8 @@ impl Client {
     }
 
     /// Retries (promotes or reattaches) a message for provided message id until it's included (referenced by a
-    /// milestone). Default interval is 5 seconds and max attempts is 40. Returns reattached messages
+    /// milestone). Default interval is 5 seconds and max attempts is 40. Returns the included message at first position
+    /// and additional reattached messages
     pub async fn retry_until_included(
         &self,
         message_id: &MessageId,
@@ -1334,10 +1280,28 @@ impl Client {
             sleep(Duration::from_secs(interval.unwrap_or(5))).await;
             // Check inclusion state for each attachment
             let message_ids_len = message_ids.len();
+            let mut conflicting = false;
             for (index, msg_id) in message_ids.clone().iter().enumerate() {
                 let message_metadata = self.get_message().metadata(msg_id).await?;
-                if message_metadata.ledger_inclusion_state.is_some() {
-                    return Ok(messages_with_id);
+                if let Some(inclusion_state) = message_metadata.ledger_inclusion_state {
+                    match inclusion_state {
+                        LedgerInclusionStateDto::Included | LedgerInclusionStateDto::NoTransaction => {
+                            // if original message, request it so we can return it on first position
+                            if message_id == msg_id {
+                                let mut included_and_reattached_messages =
+                                    vec![(*message_id, self.get_message().data(message_id).await?)];
+                                included_and_reattached_messages.extend(messages_with_id);
+                                return Ok(included_and_reattached_messages);
+                            } else {
+                                // Move included message to first position
+                                messages_with_id.rotate_left(index);
+                                return Ok(messages_with_id);
+                            }
+                        }
+                        // only set it as conflicting here and don't return, because another reattached message could
+                        // have the included transaction
+                        LedgerInclusionStateDto::Conflicting => conflicting = true,
+                    };
                 }
                 // Only reattach or promote latest attachment of the message
                 if index == message_ids_len - 1 {
@@ -1352,6 +1316,17 @@ impl Client {
                     }
                 }
             }
+            // After we checked all our reattached messages, check if the transaction got reattached in another message
+            // and confirmed
+            if conflicting {
+                let message = self.get_message().data(message_id).await?;
+                if let Some(Payload::Transaction(transaction_payload)) = message.payload() {
+                    let included_message = self.get_included_message(&transaction_payload.id()).await?;
+                    let mut included_and_reattached_messages = vec![(included_message.id().0, included_message)];
+                    included_and_reattached_messages.extend(messages_with_id);
+                    return Ok(included_and_reattached_messages);
+                }
+            }
         }
         Err(Error::TangleInclusionError(message_id.to_string()))
     }
